#!/usr/bin/env python3
try:
    from selenium.webdriver.support.ui import WebDriverWait
    from selenium.webdriver.support import expected_conditions as EC
    from selenium.webdriver.common.by import By
    from selenium.common.exceptions import NoSuchElementException,WebDriverException
    from random import randint
    from selenium.webdriver.common.keys import Keys
    import sys
except Exception as ex:
    print(ex)

class Utilities:

    @staticmethod
    def __close_driver(driver):
        """expects driver's instance, closes the driver"""
        try:
            driver.close()
            driver.quit()
        except Exception as ex:
            print("error at close_driver method : {}".format(ex))

    @staticmethod
    def __close_error_popup(driver):
        '''expects driver's instance as a argument and checks if error shows up
        like "We could not process your request. Please try again later" ,
        than click on close button to skip that popup.'''
        try:
            WebDriverWait(driver, 10).until(EC.element_to_be_clickable((By.CSS_SELECTOR,'a.layerCancel'))) #wait for popup to show
            button = driver.find_element_by_css_selector("a.layerCancel") #grab that popup's close button
            button.click()  #click "close" button
        except WebDriverException:
            #it is possible that even after waiting for given amount of time,modal may not appear
            pass
        except NoSuchElementException:
            pass  #passing this error silently because it may happen that popup never shows up

        except Exception as ex:
            #if any other error occured except the above one
            print("error at close_error_popup method : {}".format(ex))

    @staticmethod
    def __scroll_down_half(driver):
        try:
          driver.execute_script("window.scrollTo(0, document.body.scrollHeight / 2);")
        except Exception as ex:
            #if any error occured than close the driver and exit
            Utilities.__close_driver(driver)
            print("error at scroll_down_half method : {}".format(ex))

    @staticmethod
    def __scroll_down(driver,layout):
        """expects driver's instance as a argument, and it scrolls down page to the most bottom till the height"""
        try:
          if layout == "old":
            driver.execute_script(
                  "window.scrollTo(0, document.body.scrollHeight);")
          elif layout == "new":
            body = driver.find_element_by_css_selector("body")
            for _ in range(randint(2, 3)):
              body.send_keys(Keys.PAGE_DOWN)
        except Exception as ex:
            #if any error occured than close the driver and exit
            Utilities.__close_driver(driver)
            print("error at scroll_down method : {}".format(ex))

    @staticmethod
    def __close_popup(driver):
        """expects driver's instance and closes modal that ask for login, by clicking "Not Now" button """
        try:
            Utilities.__scroll_down_half(driver)  #try to scroll
            #wait for popup to show
            WebDriverWait(driver, 10).until(EC.element_to_be_clickable((By.ID,'expanding_cta_close_button')))
            #grab "Not Now" button
            popup_close_button = driver.find_element_by_id('expanding_cta_close_button')
            popup_close_button.click()  #click the button
        except WebDriverException:
            #modal may not popup, so no need to raise exception in case it is not found
            pass
        except NoSuchElementException:
            pass  #passing this exception silently as modal may not show up
        except Exception as ex:
            print("error at close_popup method : {}".format(ex))

    @staticmethod
    def __wait_for_element_to_appear(driver,layout):
        """expects driver's instance, wait for posts to show.
        post's CSS class name is userContentWrapper
        """
        try:
            if layout == "old":
              #wait for page to load so posts are visible
              body = driver.find_element_by_css_selector("body")
              for _ in range(randint(3, 5)):
                body.send_keys(Keys.PAGE_DOWN)
              WebDriverWait(driver, 30).until(EC.presence_of_element_located((By.CSS_SELECTOR,'.userContentWrapper')))
            elif layout == "new":
              WebDriverWait(driver, 30).until(
                  EC.presence_of_element_located((By.CSS_SELECTOR, "[aria-posinset]")))

        except WebDriverException:
            #if it was not found,it means either page is not loading or it does not exists
            print("No posts were found!")
            Utilities.__close_driver(driver)
            sys.exit(1)  #exit the program, because if posts does not exists,we cannot go further
        except Exception as ex:
            print("error at wait_for_element_to_appear method : {}".format(ex))
            Utilities.__close_driver(driver)



    @staticmethod
    def __click_see_more(driver,content):
        """expects driver's instance and selenium element, click on "see more" link to open hidden content"""
        try:
            #find element and click 'see more' button
            element = content.find_element_by_css_selector('span.see_more_link_inner')
            driver.execute_script("arguments[0].click();", element) #click button using js

        except NoSuchElementException:
            #if it doesn't exists than no need to raise any error
            pass
        except AttributeError:
            pass
        except IndexError:
            pass
        except Exception as ex:
            print("error at click_see_more method : {}".format(ex))


#!/usr/bin/env python3
try:
    from selenium.common.exceptions import NoSuchElementException
    from .scraping_utilities import Scraping_utilities
    from .driver_utilities import Utilities
    import sys
    import urllib.request
    import re
    from dateutil.parser import parse
    import dateutil
    import datetime
except Exception as ex:
    print(ex)

class Finder():
    """
    Holds the collections of methods that finds element of the facebook's posts using selenium's webdriver's methods
    """
    @staticmethod
    def __get_status_link(link_list):
      status = ""
      for link in link_list:
        link_value = link.get_attribute("href")
        if "/posts/" in link_value and "/groups/" in link_value:
            status = link
            break
        if "/posts/" in link_value:
            status = link
            break
        if "/videos/pcb" in link_value:
            status = link
            break
        elif "/photos/" in link_value:
            # print(link_value)
            status = link
            break
        if "fbid=" in link_value:
            status = link
            break
        elif "/group/" in link_value:
            # print(link_value)
            status = link
            break
        if "/videos/" in link_value:
            # print(link_value)
            status = link
            break
        elif "/groups/" in link_value:
            # print(link_value)
            status = link
            break
      return status

    @staticmethod
    def __find_status(post,layout):
        """finds URL of the post, then extracts link from that URL and returns it"""
        try:
            link = None
            if layout == "old":
              #aim is to find element that looks like <a href="URL" class="_5pcq"></a>
              #after finding that element, get it's href value and pass it to different method that extracts post_id from that href
              status_link = post.find_element_by_class_name("_5pcq").get_attribute("href")
              #extract out post id from post's url
              status = Scraping_utilities._Scraping_utilities__extract_id_from_link(status_link)
            elif layout == "new":
              links = post.find_elements_by_css_selector("a[role='link']")
              link = Finder.__get_status_link(links)
              status_link = link.get_attribute('href')
              print("Status Link: ",status_link)
              status = Scraping_utilities._Scraping_utilities__extract_id_from_link(
                  status_link)
        except NoSuchElementException:
            #if element is not found
            status = "NA"

        except Exception as ex:
            print("error at find_status method : {}".format(ex))
            status = "NA"

        return (status,status_link,link)

    @staticmethod
    def __find_share(post,layout):
        """finds shares count of the facebook post using selenium's webdriver's method"""
        try:
            if layout == "old":
              #aim is to find element that have datatest-id attribute as UFI2SharesCount/root
              shares = post.find_element_by_css_selector("[data-testid='UFI2SharesCount/root']").get_attribute('textContent')
              shares = Scraping_utilities._Scraping_utilities__extract_numbers(shares)
            elif layout == "new":
              elements = post.find_elements_by_css_selector("div.gtad4xkn")
              shares = "0"
              for element in elements:
                text = element.text
                if "share" in text:
                    shares = re.findall("\d+", text)[0]
                    break
            return shares
        except NoSuchElementException:
            #if element is not present that means there wasn't any shares
            shares = 0

        except Exception as ex:
            print("error at find_share method : {}".format(ex))
            shares = 0


        return shares

    @staticmethod
    def __find_reactions(post):
        """finds all reaction of the facebook post using selenium's webdriver's method"""
        try:
            #find element that have attribute aria-label as 'See who reacted to this
            reactions_all = post.find_element_by_css_selector(
                '[aria-label="See who reacted to this"]')
        except NoSuchElementException:
            reactions_all = ""
        except Exception as ex:
            print("error at find_reactions method : {}".format(ex))
        return reactions_all

    @staticmethod
    def __find_comments(post,layout):
        """finds comments count of the facebook post using selenium's webdriver's method"""
        try:
            comments = ""
            if layout == "old":
              comments = post.find_element_by_css_selector("a._3hg-").get_attribute('textContent')
              #extract numbers from text
              comments = Scraping_utilities._Scraping_utilities__extract_numbers(comments)
            elif layout == "new":
              elements = post.find_elements_by_css_selector("div.gtad4xkn")
              comments = "0"
              for element in elements:
                text = element.text
                if "comment" in text:
                    comments = Scraping_utilities._Scraping_utilities__extract_numbers(
                        comments)
        except NoSuchElementException:
            comments = 0
        except Exception as ex:
            print("error at find_comments method : {}".format(ex))
            comments = 0

        return comments

    @staticmethod
    def __fetch_post_passage(href):

        response = urllib.request.urlopen(href)

        text = response.read().decode('utf-8')

        post_message_div_finder_regex = '<div data-testid="post_message" class=".*?" data-ft=".*?">(.*?)<\/div>'

        post_message = re.search(post_message_div_finder_regex,text)

        replace_html_tags_regex = '<[^<>]+>'
        message = re.sub(replace_html_tags_regex,'',post_message.group(0))

        return message

    @staticmethod
    def __element_exists(element,css_selector):
        try:
            found = element.find_element_by_css_selector(css_selector)
            return True
        except NoSuchElementException:
            return False

    @staticmethod
    def __find_content(post,driver,layout):
        """finds content of the facebook post using selenium's webdriver's method and returns string containing text of the posts"""
        try:
            if layout == "old":
              post_content = post.find_element_by_class_name('userContent')
            elif layout == "new":
              post_content = post.find_element_by_css_selector('[data-ad-preview="message"]')
            #if 'See more' or 'Continue reading' is present in post
            if Finder._Finder__element_exists(post_content,"span.text_exposed_link > a"):
                element = post_content.find_element_by_css_selector("span.text_exposed_link > a") #grab that element
                #if element have already the onclick function, that means it is expandable paragraph
                if element.get_attribute("onclick"):
                    Utilities._Utilities__click_see_more(driver,post_content) #click 'see more' button to get hidden text as well
                    content = Scraping_utilities._Scraping_utilities__extract_content(post_content) #extract content out of it
                elif element.get_attribute("target"): #if element have attribute of target="_blank"
                #if it does not have onclick() method, it means we'll to extract passage by request
                #if content have attribute target="_blank" it indicates that text will open in new tab,
                #so make a seperate request and get that text
                    content = Finder._Finder__fetch_post_passage(element.get_attribute("href"))
            else:
                #if it does not have see more, just get the text out of it
                content = post_content.get_attribute("textContent")

        except NoSuchElementException:
            #if [data-testid="post_message"] is not found, it means that post did not had any text,either it is image or video
            content = ""
        except Exception as ex:
            print("error at find_content method : {}".format(ex))
            content = ""
        return content


    @staticmethod
    def __find_posted_time(post,layout,link_element):
        """finds posted time of the facebook post using selenium's webdriver's method"""
        try:
            #extract element that looks like <abbr class='_5ptz' data-utime="some unix timestamp"> </abbr>
            #posted_time = post.find_element_by_css_selector("abbr._5ptz").get_attribute("data-utime")
            if layout == "old":
              posted_time = post.find_element_by_tag_name("abbr").get_attribute('data-utime')
              return datetime.datetime.fromtimestamp(float(posted_time)).isoformat()
            elif layout == "new":
              aria_label_value = link_element.get_attribute("aria-label")
              timestamp = parse(aria_label_value).isoformat() if len(
                  aria_label_value) > 5 else Scraping_utilities._Scraping_utilities__convert_to_iso(aria_label_value)
              return timestamp
        except dateutil.parser._parser.ParserError:
            timestamp = Scraping_utilities._Scraping_utilities__convert_to_iso(
                aria_label_value)
            return timestamp
        except TypeError:
            timestamp = ""
        except Exception as ex:
            print("error at find_posted_time method : {}".format(ex))
            timestamp = ""
            return timestamp


    @staticmethod
    def __find_video_url(post,page_name,status):
        """finds video of the facebook post using selenium's webdriver's method"""
        try:
            #if video is found in the post, than create a video URL by concatenating post's id with page_name
            video_element = post.find_element_by_tag_name("video")
            video = "https://www.facebook.com/{}/videos/{}".format(page_name,status)

        except NoSuchElementException:
            video = ""
            pass
        except Exception as ex:
            video = ""
            print("error at find_video_url method : {}".format(ex))

        return video

    @staticmethod
    def __find_image_url(post):
        """finds all image of the facebook post using selenium's webdriver's method"""
        try:
            #find all img tag that looks like <img class="scaledImageFitWidth img" src="">
            images = post.find_elements_by_css_selector("img.scaledImageFitWidth.img")
            #extract src attribute from all the img tag,store it in list
            sources = [image.get_attribute("src") for image in images] if len(images) > 0 else []
        except NoSuchElementException:
            sources = []
            pass
        except Exception as ex:
            print("error at find_image_url method : {}".format(ex))
            sources = []

        return sources

    @staticmethod
    def __find_all_posts(driver,layout):
        """finds all posts of the facebook page using selenium's webdriver's method"""
        try:
            #find all posts that looks like <div class="userContentWrapper"> </div>
            if layout == "old":
              all_posts = driver.find_elements_by_css_selector("div.userContentWrapper")
            elif layout == "new":
              all_posts = driver.find_elements_by_css_selector(
                  '[aria-posinset]')
            return all_posts
        except NoSuchElementException:
            print("Cannot find any posts! Exiting!")
            #if this fails to find posts that means, code cannot move forward, as no post is found
            Utilities.__close_driver(driver)
            sys.exit(1)
        except Exception as ex:
            print("error at __find_all_posts method : {}".format(ex))
            Utilities.__close_driver(driver)
            sys.exit(1)

    @staticmethod
    def __find_name(driver,layout):
        """finds name of the facebook page using selenium's webdriver's method"""
        try:
            if layout == "old":
              name =  driver.find_element_by_css_selector('a._64-f').get_attribute('textContent')
            elif layout == "new":
              name = driver.find_element_by_tag_name("strong").get_attribute("textContent")
            return name
        except Exception as ex:
            print("error at __find_name method : {}".format(ex))

    @staticmethod
    def __detect_ui(driver):
      try:
        driver.find_element_by_id("pagelet_bluebar")
        return "old"
      except NoSuchElementException:
        return "new"
      except Exception as ex:
        print("error art __detect_ui: {}".format(ex))
        Utilities.__close_driver(driver)
        sys.exit(1)

    @staticmethod
    def __find_reaction(layout, reactions_all):
      try:
        if layout == "old":
          return reactions_all.find_elements_by_tag_name(
              "a")
        elif layout == "new":
          return reactions_all.find_elements_by_tag_name(
            "div")

      except Exception as ex:
        print("find_reaction", ex)
        return ""




#!/usr/bin/env python3
try:
    from seleniumwire import webdriver
    # to add capabilities for chrome and firefox, import their Options with different aliases
    from selenium.webdriver.chrome.options import Options as ChromeOptions
    from selenium.webdriver.firefox.options import Options as FirefoxOptions
    # import webdriver for downloading respective driver for the browser
    from webdriver_manager.chrome import ChromeDriverManager
    from webdriver_manager.firefox import GeckoDriverManager
except Exception as ex:
    print(ex)


class Initializer:

    def __init__(self, browser_name, proxy=None):
        self.browser_name = browser_name
        self.proxy = proxy

    def set_properties(self, browser_option):
        """adds capabilities to the driver"""
        browser_option.add_argument(
            '--headless')  # runs browser in headless mode
        browser_option.add_argument('--no-sandbox')
        browser_option.add_argument("--disable-dev-shm-usage")
        browser_option.add_argument('--ignore-certificate-errors')
        browser_option.add_argument('--disable-gpu')
        browser_option.add_argument('--log-level=3')
        browser_option.add_argument('--disable-notifications')
        browser_option.add_argument('--disable-popup-blocking')

        # browser_option.add_argument(
        #   "--proxy-server=http://{}".format(self.proxy.replace(" ", "")))

        return browser_option

    def set_driver_for_browser(self, browser_name):
        """expects browser name and returns a driver instance"""
        # if browser is suppose to be chrome
        if browser_name.lower() == "chrome":
            browser_option = ChromeOptions()
            # automatically installs chromedriver and initialize it and returns the instance
            if self.proxy is not None:
                options = {
                    'https': 'https://{}'.format(self.proxy.replace(" ", "")),
                    'http': 'http://{}'.format(self.proxy.replace(" ", "")),
                    'no_proxy': 'localhost, 127.0.0.1'
                }
                print("Using: {}".format(self.proxy))
                return webdriver.Chrome(executable_path=ChromeDriverManager().install(),
                                        options=self.set_properties(browser_option), seleniumwire_options=options)

            return webdriver.Chrome(executable_path=ChromeDriverManager().install(), options=self.set_properties(browser_option))
        elif browser_name.lower() == "firefox":
            browser_option = FirefoxOptions()
            if self.proxy is not None:
                options = {
                    'https': 'https://{}'.format(self.proxy.replace(" ", "")),
                    'http': 'http://{}'.format(self.proxy.replace(" ", "")),
                    'no_proxy': 'localhost, 127.0.0.1'
                }
                print("Using: {}".format(self.proxy))
                return webdriver.Firefox(executable_path=GeckoDriverManager().install(),
                                         options=self.set_properties(browser_option), seleniumwire_options=options)

            # automatically installs geckodriver and initialize it and returns the instance
            return webdriver.Firefox(executable_path=GeckoDriverManager().install(), options=self.set_properties(browser_option))
        else:
            # if browser_name is not chrome neither firefox than raise an exception
            raise Exception("Browser not supported!")

    def init(self):
        """returns driver instance"""
        driver = self.set_driver_for_browser(self.browser_name)
        return driver



import setuptools

with open("README.MD","r") as file:
    long_description = file.read()

requirements = []

for line in open("requirements.txt",'r',encoding="utf-8").readlines():
  requirements.append(line.replace("\n",""))

setuptools.setup(
    name = "facebook_page_scraper",
    version = "0.1.10",
    author = "Sajid Shaikh",
    author_email = "shaikhsajid3732@gmail.com",
    description = "Python package to scrap facebook's pages front end with no limitations",
    long_description = long_description,
    long_description_content_type = "text/markdown",
    license="MIT",
    url = "https://github.com/shaikhsajid1111/facebook_page_scraper",
    keywords = "web-scraping selenium facebook facebook-pages",
    packages = setuptools.find_packages(),
    classifiers = [
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: Implementation :: CPython",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX :: Linux",
        "Operating System :: MacOS",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Internet :: WWW/HTTP"

    ],
    python_requires = ">=3.6",
    install_requires=requirements
)
